part of barcodes;

/// Represents the symbology
abstract class Symbology {
  /// Creates the symbology
  Symbology({this.module})
      : assert((module != null && module > 0) || module == null);

  /// Specifies the size of the smallest line or dot of the code.
  /// The property is measured in a logical  pixel.
  /// Define anyone of barcode symbology that will be used to convert
  /// input value into visual barcode representation.
  /// ```dart
  /// Widget build(BuildContext context) {
  ///    return Container(
  ///        child: SfBarcodeGenerator(value:'123456',
  ///        symbology: UPCE(module: 2)));
  ///}
  /// ```dart
  final int module;

  /// Specifies the value with start and the stop symbol
  String _valueWithStartAndStopSymbol;

  /// Specifies the text size
  Size _textSize;

  /// Method to valid whether the provided input character is supported
  /// by corresponding symbology
  bool _validateInput(String value);

  /// Method to render the barcode value
  void _renderBarcode(
      Canvas canvas,
      Size size,
      Offset offset,
      String value,
      Color foregroundColor,
      TextStyle textStyle,
      double textSpacing,
      TextAlign textAlign,
      bool showValue);

  /// Renders the paint for the bar code
  Paint _getBarPaint(Color foregroundColor) {
    return Paint()
      ..color = foregroundColor
      ..strokeWidth = 1
      ..style = PaintingStyle.fill;
  }

  /// Calculates the left value of the initial bar code
  double _calculateLeftPosition(
      int barWidth, int module, double width, double offsetX) {
    final int _calculatedWidth = barWidth * module;
    double _diffInWidth = (width - _calculatedWidth) / 2;
    _diffInWidth += offsetX;
    return _diffInWidth;
  }

  /// Method to render the input value of the barcode
  void _drawText(Canvas canvas, Offset offset, Size size, String value,
      TextStyle textStyle, double textSpacing, TextAlign textAlign) {
    final TextSpan _span = TextSpan(text: value, style: textStyle);

    final TextPainter _textPainter = TextPainter(
        text: _span, textDirection: TextDirection.ltr, textAlign: textAlign);
    _textPainter.layout();
    double _x;
    double _y;
    switch (textAlign) {
      case TextAlign.justify:
      case TextAlign.center:
        {
          _x = (offset.dx + size.width / 2) - _textSize.width / 2;
          _y = offset.dy + size.height + textSpacing;
        }
        break;
      case TextAlign.left:
      case TextAlign.start:
        {
          _x = offset.dx;
          _y = offset.dy + size.height + textSpacing;
        }
        break;
      case TextAlign.right:
      case TextAlign.end:
        {
          _x = offset.dx + (size.width - _textSize.width);
          _y = offset.dy + size.height + textSpacing;
        }
        break;
    }
    _textPainter.paint(canvas, Offset(_x, _y));
  }
}
